'''
Created on Oct 20, 2011

@author: lucas
'''

import hashlib

class Detector(object):
    '''
    local implementation of Winnowing technique. Processes natural language text, and returns suspicious pairs.
    '''

    def __init__(self, fileList):
        '''
        Constructor
        '''
        self.files = fileList
        self.data = {}
        self.results = {}
    
    def runForAll(self):
        try:
            for f in self.files:
                original = open(f, 'r').read()
                clean, map = self.cleanAndMapText(f)
                fingerprints = self.winnow(clean, 100, 50)
                self.data[f] = [original, clean, map, fingerprints]
            #2 by 2 fingerprint comparison
            for i in self.data:
                fingerA = self.data[i][3]
                for j in self.data:
                    fingerB = self.data[j][3]
                    if i < j:
                        shared = self.compareFingerPrints(fingerA, fingerB)
                        if shared != []:
                            sub = self.findSubstring(shared, self.data[i][3], self.data[i][2], self.data[i][0])
                            self.results[i, j] = sub
        except Exception as e:
            print 'Error on local detection! ', e
            raise e
        return self.results
    
    def cleanAndMapText(self, textFile):
        '''
        reads a textFile, removes irrelevant chars, and passes to lower case. Creates a mapping (position in cleanString) to (position in original text) 
        '''
        inputText = open(textFile, 'r').read()
        cleanText = ''
        positionMap = []
        irrelevantChars = [' ', ',', '\n', '.', '\t', '=', '-', ':']
        for i in range(0, len(inputText)):
            if inputText[i] not in irrelevantChars:
                cleanText += inputText[i].lower()
                positionMap.append(i)
        return cleanText, positionMap
    
    def winnow(self, text, windowSize, k):
        '''
        Local implementation of Winnowing technique.
        '''
        rawHashes = []      #k-grams hashes
        fingerPrints = {}   #saves hash and position
        #computes hash for each k-gram in the text
        for i in range(0, len(text) - k + 1):
            rawHashes.append(hashlib.md5(text[i:i+k]).hexdigest())
        min = 0             #last selected hash position
        #given a window length w, picks a hash
        for i in range(len(rawHashes) - windowSize + 1):
            window = rawHashes[i:i+windowSize]
            for j in range(0, len(window)):
                if window[j] <= rawHashes[min] or min < i:
                    min = i+j
            fingerPrints[rawHashes[min]] = min
        return fingerPrints
    
    def compareFingerPrints(self, fingerPrintsA, fingerPrintsB):
        '''
        compares two fingerprint sets. Returns common fingerprints (possible plagiarism).
        '''
        a = set(fingerPrintsA.keys())
        b = set(fingerPrintsB.keys())
        sharedFingerPrints = list(a.intersection(b))
        return sharedFingerPrints
    
    def findSubstring(self, sharedFingerPrints, fingerPrints, map, text):
        '''
        finds substring of sharedFingerPrints in original text
        '''
        positions = []
        for f in sharedFingerPrints:
            positions.append(fingerPrints[f])
        if positions != []:    
            start = map[min(positions)]
            end = map[max(positions)]
            return text[start:end]
        else:
            return ''