'''
Created on Sep 5, 2011

@author: lucas
'''
import networkx as nx
import matplotlib.pyplot as plt

class Graph():
    '''
    generates the graph for the report
    '''
    
    def __init__(self):
        self.G = nx.Graph()

    def buildGraphFromMossResults(self, aMossResultList, aLocalResultDict):
        '''
        returns a file with a graph that shows MOSS results list
        '''
        for result in aMossResultList:
            a = result[0][0]
            b = result[1][0]
            if a in self.G.nodes():
                self.G.node[a]['size'] =  self.G.node[a]['size'] * (float(result[0][1])/100 + 1)
            else:
                self.G.add_node(a, size = (float(result[0][1])/100 + 1))
            if b in self.G.nodes():
                self.G.node[b]['size'] =  self.G.node[b]['size'] * (float(result[1][1])/100 + 1)
            else:
                self.G.add_node(b, size = (float(result[0][1])/100 + 1))
            self.G.add_edge(a, b)
            self.G.edge[a][b]['link'] = result[0][2]   
        for local in aLocalResultDict:
            a = local[0]
            b = local[1]
            if not local[0] in self.G.nodes():
                self.G.add_node(local[0], size = 1.5)
            if not local[1] in self.G.nodes():
                self.G.add_node(local[1], size = 1.5)
            self.G.add_edge(local[0], local[1])
        
    def drawImageToFile(self, aFilename):
        '''
        writes the graph image to aFilename
        '''
        plt.rcParams['text.usetex'] = False
        plt.figure(figsize=(8,8))
        """
        nx.draw_networkx_edges(H,pos,alpha=0.3,width=edgewidth, edge_color='m')
        """
        nodesize = []
        baseSize = 1200
        size = 1
        for v in self.G.nodes():
            if self.G.node[v]['size'] < 5:
                size = self.G.node[v]['size']*baseSize
            else: #limits maximum node display size
                size = 5
            nodesize.append(size)
        pos = nx.graphviz_layout(self.G)
        nx.draw_networkx_nodes(self.G,pos,node_size=nodesize,node_color='r',alpha=0.8)
        nx.draw_networkx_edges(self.G,pos,alpha=0.4,node_size=0,width=1,edge_color='k')
        nx.draw_networkx_labels(self.G, pos, fontsize=20)
        font = {'fontname'   : 'Bitstream Vera Sans', 'color' : 'k','fontweight' : 'bold','fontsize'   : 14}
        plt.title("Similarity Graph", font)
        plt.axis('off') 
        plt.savefig(aFilename)