from pyparsing import *

"""
this is a parser for /edit style pages. Much easier.
"""

import authenticator
import mechanize
import BeautifulSoup
from pyparsing import *
import re

class Parser():
    
    def __init__(self, anAuthenticator):
        self.br = mechanize.Browser(factory=mechanize.RobustFactory()) # este parametro eh FUNDAMENTAL para detectar os campos corretos...vem do beautiful soup vide http://wwwsearch.sourceforge.net/mechanize/faq.html 
        self.cj = mechanize.LWPCookieJar() 
        self.br.set_cookiejar(anAuthenticator.cj)
        self.contents, self.pythonCode, self.cppCode, self.latexCode, self.nlText = None, None, None, None, None
        self.dictionary = {}
        self.dictionary['python'], self.dictionary['latex'], self.dictionary['cpp'], self.dictionary['nlText'] = '', '', '', ''
        
    def extractContents(self, aUrl):
        """
        takes in /view/ aUrl and extract the (user-created) contents from the matching /edit/ url
        """
        try:
            view = Regex('/view/$')
            view.setParseAction(replaceWith('/edit/'))
            editUrl = view.transformString(aUrl)
            self.br.open(editUrl)
            self.br.select_form('edit_form')
            self.contents = self.br.form.get_value('dok')
        except Exception as e:
            print 'Error when processing ', editUrl, '\nDescription: ', e
            return None
        return self.contents
                
    def parseContents(self):
        """
        detects code blocks and NL text blocks  in self.contents. Identation matters. Code must be at least (but not necessarily) 1 indentation level ahead of code declaration line.        
        """
        language = ''
        for contentLine in self.contents.splitlines():
            codeBlock = None
            codeOption = Literal(':') + Word(alphanums + '_') + Literal (':') + Word(alphanums)
            baseIndentLevel = 0
            if self.detectCodeLanguage(contentLine) != None:
                language = self.detectCodeLanguage(contentLine)
                baseIndentLevel = len(re.match("^(\s)*", contentLine).group()) / 4
            else:
                try:
                    if codeOption.parseString(contentLine):
                        pass
                except:
                    try:
                        currentIndentLevel = len(re.match("^(\s)*", contentLine).group()) / 4
                        if currentIndentLevel > baseIndentLevel: #indentation means code
                            self.dictionary[language] += (contentLine + '\n')
                        else: #unindentation means NL text
                            self.dictionary['nlText'] += (contentLine + '\n')
                    except: #bad formatted NL text, no problem.
                        self.dictionary['nlText'] += (contentLine + '\n')
        return self.contents        
    
    def detectCodeLanguage(self, aLine):
        codeDef = (Literal('.. code::') | Literal('.. algorithm::')) + Word(alphas)
        try:
            language = codeDef.parseString(aLine)[1]
            return language
        except Exception as e:
            return None
            
    def zeroPass(self, aReadPage):
        """
        helper for link extraction method.
        """
        removeText = replaceWith("")
        scriptOpen,scriptClose = makeHTMLTags("script")
        scriptBody = scriptOpen + SkipTo(scriptClose) + scriptClose
        scriptBody.setParseAction(removeText)
        headOpen,headClose = makeHTMLTags("head")
        headBody = headOpen + SkipTo(headClose) + headClose
        headBody.setParseAction(removeText)
        shOpen = '<div class="showhide">'
        shClose = '</div>'
        shBody = shOpen + SkipTo(shClose) + shClose
        shBody.setParseAction(removeText)
        divStyleOpen = '<div style="white-space: nowrap;">'
        divStyleClose = '</div>'
        divStyleBody = divStyleOpen + SkipTo(divStyleClose) + divStyleClose
        divStyleBody.setParseAction(removeText)
        cssStyleOpen, cssStyleClose = makeHTMLTags("style")
        cssStyleBody = cssStyleOpen + SkipTo(cssStyleClose) + cssStyleClose
        cssStyleBody.setParseAction(removeText)
        tabsOpen, tabsClose = '<div class="tabs">', '</div>'
        tabsBody = tabsOpen + SkipTo(tabsClose) + tabsClose
        tabsBody.setParseAction(removeText)
        barOpen, barClose = '<div class="personalBar">', '</div>'
        barBody = barOpen + SkipTo(barClose) + barClose
        barBody.setParseAction(removeText) 
        pathOpen, pathClose = '<div class="pathBar">', '</div>'
        pathBody = pathOpen + SkipTo(pathClose) + pathClose
        pathBody.setParseAction(removeText)
        boxOpen, boxClose = '<div class="box">', '</div>'
        boxBody = boxOpen + SkipTo(boxClose) + boxClose
        boxBody.setParseAction(removeText)
        commentOpen,commentClose = '<div id="comments">', '</div>'
        commentBody = commentOpen + SkipTo(commentClose) + commentClose
        commentBody.setParseAction(removeText)
        formCommentOpen,formCommentClose = '<div id="commentform">', '</div>'
        formCommentBody = formCommentOpen + SkipTo(formCommentClose) + formCommentClose
        formCommentBody.setParseAction(removeText)
        headerOpen, headerClose = '<div id="header">', '</div>'
        headerBody = headerOpen + SkipTo(headerClose) + headerClose
        headerBody.setParseAction(removeText)
        footerOpen, footerClose = '<div id="footer">', '</div>'
        footerBody = footerOpen + SkipTo(footerClose) + footerClose
        footerBody.setParseAction(removeText)
        htmlComment.setParseAction(removeText)
        commonHTMLEntity.setParseAction(replaceHTMLEntity)
        docOpen = '<!DOCTYPE'
        docClose = '\">'
        docBody = docOpen + SkipTo(docClose) + docClose
        docBody.setParseAction(removeText)
        singleQuotes = Literal("&#39;")     #replaces doubleQuotes (read in html as &#39;)
        singleQuotes.setParseAction(replaceWith("\'"))
        # first pass, strip out tags and translate entities
        zeroPass = (docBody | headBody| headerBody | htmlComment | scriptBody | shBody | divStyleBody | cssStyleBody | tabsBody | barBody | pathBody | boxBody | commentBody | formCommentBody | footerBody | commonHTMLEntity | singleQuotes).transformString(aReadPage)
        return zeroPass
    
    def extractLinks(self, aReadPage):
        """
        Method for link extraction. Links are expected to be relative (/adesso/wiki/view), not full (http://www.usp.br).
        Must be used, because mechanize's browser doesn't preserve the links positioning in the page.
        """
        firstPass = self.zeroPass(aReadPage)
        linkList = []
        aStart,aEnd = makeHTMLTags("A")
        link = aStart + SkipTo(aEnd)("link") + aEnd
        link.ignore(htmlComment) 
        for toks,start,end in link.scanString(firstPass):
            linkList.append(toks.startA.href)
        return linkList
    
    def parseMossResults(self, aReadPage):
        removeText = replaceWith("")
        tableStart, tableEnd = makeHTMLTags("table")
        tableBody = tableStart + SkipTo(tableEnd) + tableEnd
        tableStart.setParseAction(removeText)
        tableEnd.setParseAction(removeText)
        tableBody.setParseAction(lambda t: str(t[1]))
        tableText = tableBody.searchString(aReadPage)[0][0]
        rowStart = Literal("<TD>")
        rowEnd = Literal("</A>")
        rowBody = rowStart + SkipTo(rowEnd) + rowEnd #each file is a row 
        rowBody.setParseAction(lambda t: str(t[1]))
        scoreStart = Literal("<TD ALIGN=right>")
        scoreEnd = LineEnd()
        scoreBody = scoreStart + SkipTo(scoreEnd) + scoreEnd
        scoreBody.setParseAction(lambda t: str(t[1]))
        rows = rowBody.searchString(tableText)
        score = scoreBody.searchString(tableText)
        results = []
        if rows.__len__() ==  2*score.__len__() and rows.__len__() > 0: #(2 rows for each score), at least one row
            for i in range(0, score.__len__()):
                results.append((self.parseMossRow(rows[2*i][0]), self.parseMossRow(rows[2*i+1][0])))
        return results
                
    def parseMossRow(self, aRow):
        expression = Literal('<A HREF=\"') + CharsNotIn('\"') + Literal('\">') + CharsNotIn('(') + Literal('(') + CharsNotIn('%') + Literal('%)')
        data = expression.parseString(aRow)
        return (data[3], data[5], data[1])